# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.entrypointfield import Entrypointfield
from ..types.file_content_ftype import FileContentFtype
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payabli_api_response_generic_2_part import PayabliApiResponseGeneric2Part
from ..types.payabli_pages import PayabliPages
from ..types.settings_query_record import SettingsQueryRecord
from .types.get_basic_entry_by_id_response import GetBasicEntryByIdResponse
from .types.get_basic_entry_response import GetBasicEntryResponse
from .types.get_entry_config_response import GetEntryConfigResponse
from .types.migrate_paypoint_response import MigratePaypointResponse
from .types.notification_request import NotificationRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPaypointClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_basic_entry(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBasicEntryResponse]:
        """
        Gets the basic details for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBasicEntryResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/basic/{jsonable_encoder(entry)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBasicEntryResponse,
                    parse_obj_as(
                        type_=GetBasicEntryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_basic_entry_by_id(
        self, id_paypoint: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBasicEntryByIdResponse]:
        """
        Retrieves the basic details for a paypoint by ID.

        Parameters
        ----------
        id_paypoint : str
            Paypoint ID. You can find this value by querying `/api/Query/paypoints/{orgId}`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBasicEntryByIdResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/basicById/{jsonable_encoder(id_paypoint)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBasicEntryByIdResponse,
                    parse_obj_as(
                        type_=GetBasicEntryByIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_entry_config(
        self,
        entry: str,
        *,
        entrypages: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetEntryConfigResponse]:
        """
        Gets the details for a single paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        entrypages : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetEntryConfigResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}",
            method="GET",
            params={
                "entrypages": entrypages,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetEntryConfigResponse,
                    parse_obj_as(
                        type_=GetEntryConfigResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliPages]:
        """
        Gets the details for single payment page for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliPages]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliPages,
                    parse_obj_as(
                        type_=PayabliPages,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseGeneric2Part]:
        """
        Deletes a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseGeneric2Part]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseGeneric2Part,
                    parse_obj_as(
                        type_=PayabliApiResponseGeneric2Part,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def save_logo(
        self,
        entry: str,
        *,
        f_content: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        furl: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Updates a paypoint logo.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        f_content : typing.Optional[str]
            Content of file, Base64-encoded. Ignored if furl is specified. Max upload size is 30 MB.

        filename : typing.Optional[str]
            The name of the attached file.

        ftype : typing.Optional[FileContentFtype]
            The MIME type of the file (if content is provided)

        furl : typing.Optional[str]
            Optional URL provided to show or download the file remotely

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/logo/{jsonable_encoder(entry)}",
            method="PUT",
            json={
                "fContent": f_content,
                "filename": filename,
                "ftype": ftype,
                "furl": furl,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def settings_page(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SettingsQueryRecord]:
        """
        Retrieves an paypoint's basic settings like custom fields, identifiers, and invoicing settings.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SettingsQueryRecord]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Paypoint/settings/{jsonable_encoder(entry)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SettingsQueryRecord,
                    parse_obj_as(
                        type_=SettingsQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def migrate(
        self,
        *,
        entry_point: Entrypointfield,
        new_parent_organization_id: int,
        notification_request: typing.Optional[NotificationRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MigratePaypointResponse]:
        """
        Migrates a paypoint to a new parent organization.

        Parameters
        ----------
        entry_point : Entrypointfield

        new_parent_organization_id : int
            The ID for the paypoint's new parent organization.

        notification_request : typing.Optional[NotificationRequest]
            Optional notification request object for a webhook

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MigratePaypointResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "Paypoint/migrate",
            method="POST",
            json={
                "entryPoint": entry_point,
                "newParentOrganizationId": new_parent_organization_id,
                "notificationRequest": convert_and_respect_annotation_metadata(
                    object_=notification_request, annotation=NotificationRequest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MigratePaypointResponse,
                    parse_obj_as(
                        type_=MigratePaypointResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPaypointClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_basic_entry(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBasicEntryResponse]:
        """
        Gets the basic details for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBasicEntryResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/basic/{jsonable_encoder(entry)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBasicEntryResponse,
                    parse_obj_as(
                        type_=GetBasicEntryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_basic_entry_by_id(
        self, id_paypoint: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBasicEntryByIdResponse]:
        """
        Retrieves the basic details for a paypoint by ID.

        Parameters
        ----------
        id_paypoint : str
            Paypoint ID. You can find this value by querying `/api/Query/paypoints/{orgId}`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBasicEntryByIdResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/basicById/{jsonable_encoder(id_paypoint)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBasicEntryByIdResponse,
                    parse_obj_as(
                        type_=GetBasicEntryByIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_entry_config(
        self,
        entry: str,
        *,
        entrypages: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetEntryConfigResponse]:
        """
        Gets the details for a single paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        entrypages : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetEntryConfigResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}",
            method="GET",
            params={
                "entrypages": entrypages,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetEntryConfigResponse,
                    parse_obj_as(
                        type_=GetEntryConfigResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliPages]:
        """
        Gets the details for single payment page for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliPages]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliPages,
                    parse_obj_as(
                        type_=PayabliPages,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_page(
        self, entry: str, subdomain: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseGeneric2Part]:
        """
        Deletes a payment page in a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        subdomain : str
            Payment page identifier. The subdomain value is the last portion of the payment page URL. For example, in`https://paypages-sandbox.payabli.com/513823dc10/pay-your-fees-1`, the subdomain is `pay-your-fees-1`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseGeneric2Part]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/{jsonable_encoder(entry)}/{jsonable_encoder(subdomain)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseGeneric2Part,
                    parse_obj_as(
                        type_=PayabliApiResponseGeneric2Part,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def save_logo(
        self,
        entry: str,
        *,
        f_content: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        furl: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Updates a paypoint logo.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        f_content : typing.Optional[str]
            Content of file, Base64-encoded. Ignored if furl is specified. Max upload size is 30 MB.

        filename : typing.Optional[str]
            The name of the attached file.

        ftype : typing.Optional[FileContentFtype]
            The MIME type of the file (if content is provided)

        furl : typing.Optional[str]
            Optional URL provided to show or download the file remotely

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/logo/{jsonable_encoder(entry)}",
            method="PUT",
            json={
                "fContent": f_content,
                "filename": filename,
                "ftype": ftype,
                "furl": furl,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def settings_page(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SettingsQueryRecord]:
        """
        Retrieves an paypoint's basic settings like custom fields, identifiers, and invoicing settings.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SettingsQueryRecord]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Paypoint/settings/{jsonable_encoder(entry)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SettingsQueryRecord,
                    parse_obj_as(
                        type_=SettingsQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def migrate(
        self,
        *,
        entry_point: Entrypointfield,
        new_parent_organization_id: int,
        notification_request: typing.Optional[NotificationRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MigratePaypointResponse]:
        """
        Migrates a paypoint to a new parent organization.

        Parameters
        ----------
        entry_point : Entrypointfield

        new_parent_organization_id : int
            The ID for the paypoint's new parent organization.

        notification_request : typing.Optional[NotificationRequest]
            Optional notification request object for a webhook

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MigratePaypointResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "Paypoint/migrate",
            method="POST",
            json={
                "entryPoint": entry_point,
                "newParentOrganizationId": new_parent_organization_id,
                "notificationRequest": convert_and_respect_annotation_metadata(
                    object_=notification_request, annotation=NotificationRequest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MigratePaypointResponse,
                    parse_obj_as(
                        type_=MigratePaypointResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.page_identifier import PageIdentifier


class CheckCaptureResponse(UniversalBaseModel):
    """
    Response model for check capture processing.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique ID for the check capture, to be used with the /api/MoneyIn/getpaid endpoint.
    """

    success: bool = pydantic.Field()
    """
    Indicates whether the check processing was successful.
    """

    process_date: typing_extensions.Annotated[str, FieldMetadata(alias="processDate")] = pydantic.Field()
    """
    The date and time when the check was processed (ISO 8601 format).
    """

    ocr_micr: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrMicr")] = pydantic.Field(
        default=None
    )
    """
    The OCR-extracted MICR (Magnetic Ink Character Recognition) line from the check.
    """

    ocr_micr_status: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrMicrStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Status of the MICR extraction process.
    """

    ocr_micr_confidence: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrMicrConfidence")] = (
        pydantic.Field(default=None)
    )
    """
    Confidence score for the MICR extraction (0 to 100).
    """

    ocr_account_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrAccountNumber")] = (
        pydantic.Field(default=None)
    )
    """
    The bank account number extracted from the check.
    """

    ocr_routing_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrRoutingNumber")] = (
        pydantic.Field(default=None)
    )
    """
    The bank routing number extracted from the check.
    """

    ocr_check_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrCheckNumber")] = (
        pydantic.Field(default=None)
    )
    """
    The check number extracted from the check.
    """

    ocr_check_tran_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrCheckTranCode")] = (
        pydantic.Field(default=None)
    )
    """
    The transaction code extracted from the check.
    """

    ocr_amount: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrAmount")] = pydantic.Field(
        default=None
    )
    """
    The amount extracted via OCR from the check.
    """

    ocr_amount_status: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ocrAmountStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Status of the amount extraction process.
    """

    ocr_amount_confidence: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="ocrAmountConfidence")
    ] = pydantic.Field(default=None)
    """
    Confidence score for the amount extraction (0 to 100).
    """

    amount_discrepancy_detected: typing_extensions.Annotated[bool, FieldMetadata(alias="amountDiscrepancyDetected")] = (
        pydantic.Field()
    )
    """
    Flag indicating whether there's a discrepancy between the provided amount and the OCR-detected amount.
    """

    endorsement_detected: typing_extensions.Annotated[bool, FieldMetadata(alias="endorsementDetected")] = (
        pydantic.Field()
    )
    """
    Flag indicating whether an endorsement was detected on the check.
    """

    errors: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of error messages that occurred during processing.
    """

    messages: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of informational messages about the processing.
    """

    car_lar_match_confidence: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="carLarMatchConfidence")
    ] = pydantic.Field(default=None)
    """
    Confidence score for the match between Courtesy Amount Recognition (CAR) and Legal Amount Recognition (LAR).
    """

    car_lar_match_status: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="carLarMatchStatus")
    ] = pydantic.Field(default=None)
    """
    Status of the CAR/LAR match.
    """

    front_image: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="frontImage")] = pydantic.Field(
        default=None
    )
    """
    Processed front image of the check (Base64-encoded).
    """

    rear_image: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="rearImage")] = pydantic.Field(
        default=None
    )
    """
    Processed rear image of the check (Base64-encoded).
    """

    check_type: typing_extensions.Annotated[float, FieldMetadata(alias="checkType")] = pydantic.Field()
    """
    Identifier for the type of check.
    Personal = 1
    Business = 2
    Only personal checks are supported for check capture.
    """

    reference_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="referenceNumber")] = (
        pydantic.Field(default=None)
    )
    """
    Reference number for the transaction.
    """

    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .display_property import DisplayProperty
from .enabled import Enabled
from .order import Order


class SettingElement(UniversalBaseModel):
    enabled: typing.Optional[Enabled] = None
    fields: typing.Optional[typing.List[DisplayProperty]] = pydantic.Field(default=None)
    """
    Fields to display on the reciept.
    """

    order: typing.Optional[Order] = None
    send_auto: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="sendAuto")] = pydantic.Field(
        default=None
    )
    """
    When `true`, Payabli automatically sends the receipt to the payor email address. 
    """

    send_manual: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="sendManual")] = pydantic.Field(
        default=None
    )
    """
    When `true`, you must send the reciept to the payor manually using the [/MoneyIn/sendreceipt/\{transId\}](/api-reference/moneyin/send-receipt-for-transaction) endpoint.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .. import core
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.entrypointfield import Entrypointfield
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_import import PayabliApiResponseImport

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawImportClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def import_bills(
        self, entry: str, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseImport]:
        """
        Import a list of bills from a CSV file. See the [Import Guide](/developers/developer-guides/bills-add#import-bills) for more help and an example file.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseImport]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/billsForm/{jsonable_encoder(entry)}",
            method="POST",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseImport,
                    parse_obj_as(
                        type_=PayabliApiResponseImport,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def import_customer(
        self,
        entry: Entrypointfield,
        *,
        file: core.File,
        replace_existing: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseImport]:
        """
        Import a list of customers from a CSV file. See the [Import Guide](/developers/developer-guides/entities-customers#import-customers) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (do not replace), 1 (replace). Default is 0

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseImport]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/customersForm/{jsonable_encoder(entry)}",
            method="POST",
            params={
                "replaceExisting": replace_existing,
            },
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseImport,
                    parse_obj_as(
                        type_=PayabliApiResponseImport,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def import_vendor(
        self, entry: Entrypointfield, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseImport]:
        """
        Import a list of vendors from a CSV file. See the [Import Guide](/developers/developer-guides/entities-vendors#import-vendors) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseImport]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/vendorsForm/{jsonable_encoder(entry)}",
            method="POST",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseImport,
                    parse_obj_as(
                        type_=PayabliApiResponseImport,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawImportClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def import_bills(
        self, entry: str, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseImport]:
        """
        Import a list of bills from a CSV file. See the [Import Guide](/developers/developer-guides/bills-add#import-bills) for more help and an example file.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseImport]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/billsForm/{jsonable_encoder(entry)}",
            method="POST",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseImport,
                    parse_obj_as(
                        type_=PayabliApiResponseImport,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def import_customer(
        self,
        entry: Entrypointfield,
        *,
        file: core.File,
        replace_existing: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseImport]:
        """
        Import a list of customers from a CSV file. See the [Import Guide](/developers/developer-guides/entities-customers#import-customers) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (do not replace), 1 (replace). Default is 0

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseImport]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/customersForm/{jsonable_encoder(entry)}",
            method="POST",
            params={
                "replaceExisting": replace_existing,
            },
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseImport,
                    parse_obj_as(
                        type_=PayabliApiResponseImport,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def import_vendor(
        self, entry: Entrypointfield, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseImport]:
        """
        Import a list of vendors from a CSV file. See the [Import Guide](/developers/developer-guides/entities-vendors#import-vendors) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseImport]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/vendorsForm/{jsonable_encoder(entry)}",
            method="POST",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseImport,
                    parse_obj_as(
                        type_=PayabliApiResponseImport,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.pagesize import Pagesize
from .raw_client import AsyncRawNotificationlogsClient, RawNotificationlogsClient
from .types.bulk_retry_request import BulkRetryRequest
from .types.notification_log import NotificationLog
from .types.notification_log_detail import NotificationLogDetail

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NotificationlogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawNotificationlogsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawNotificationlogsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawNotificationlogsClient
        """
        return self._raw_client

    def search_notification_logs(
        self,
        *,
        start_date: dt.datetime,
        end_date: dt.datetime,
        page_size: typing.Optional[Pagesize] = None,
        skip: typing.Optional[int] = None,
        notification_event: typing.Optional[str] = OMIT,
        succeeded: typing.Optional[bool] = OMIT,
        org_id: typing.Optional[int] = OMIT,
        paypoint_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[NotificationLog]:
        """
        Search notification logs with filtering and pagination.
          - Start date and end date cannot be more than 30 days apart
          - Either `orgId` or `paypointId` must be provided

        This endpoint requires the `notifications_create` OR `notifications_read` permission.

        Parameters
        ----------
        start_date : dt.datetime
            The start date for the search.

        end_date : dt.datetime
            The end date for the search.

        page_size : typing.Optional[Pagesize]

        skip : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        notification_event : typing.Optional[str]
            The type of notification event to filter by.

        succeeded : typing.Optional[bool]
            Indicates whether the notification was successful.

        org_id : typing.Optional[int]
            The ID of the organization to filter by.

        paypoint_id : typing.Optional[int]
            The ID of the paypoint to filter by.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[NotificationLog]

        Examples
        --------
        import datetime

        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notificationlogs.search_notification_logs(
            page_size=20,
            start_date=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            end_date=datetime.datetime.fromisoformat(
                "2024-01-31 23:59:59+00:00",
            ),
            org_id=12345,
            notification_event="ActivatedMerchant",
            succeeded=True,
        )
        """
        _response = self._raw_client.search_notification_logs(
            start_date=start_date,
            end_date=end_date,
            page_size=page_size,
            skip=skip,
            notification_event=notification_event,
            succeeded=succeeded,
            org_id=org_id,
            paypoint_id=paypoint_id,
            request_options=request_options,
        )
        return _response.data

    def get_notification_log(
        self, uuid_: uuid.UUID, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NotificationLogDetail:
        """
        Get detailed information for a specific notification log entry.
        This endpoint requires the `notifications_create` OR `notifications_read` permission.

        Parameters
        ----------
        uuid_ : uuid.UUID
            The notification log entry.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotificationLogDetail

        Examples
        --------
        import uuid

        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notificationlogs.get_notification_log(
            uuid_=uuid.UUID(
                "550e8400-e29b-41d4-a716-446655440000",
            ),
        )
        """
        _response = self._raw_client.get_notification_log(uuid_, request_options=request_options)
        return _response.data

    def retry_notification_log(
        self, uuid_: uuid.UUID, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NotificationLogDetail:
        """
        Retry sending a specific notification.

        **Permissions:** notifications_create

        Parameters
        ----------
        uuid_ : uuid.UUID
            Unique id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotificationLogDetail

        Examples
        --------
        import uuid

        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notificationlogs.retry_notification_log(
            uuid_=uuid.UUID(
                "550e8400-e29b-41d4-a716-446655440000",
            ),
        )
        """
        _response = self._raw_client.retry_notification_log(uuid_, request_options=request_options)
        return _response.data

    def bulk_retry_notification_logs(
        self, *, request: BulkRetryRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Retry sending multiple notifications (maximum 50 IDs).
        This is an async process, so use the search endpoint again to check the notification status.

        This endpoint requires the `notifications_create` permission.

        Parameters
        ----------
        request : BulkRetryRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notificationlogs.bulk_retry_notification_logs(
            request=[
                uuid.UUID(
                    "550e8400-e29b-41d4-a716-446655440000",
                ),
                uuid.UUID(
                    "550e8400-e29b-41d4-a716-446655440001",
                ),
                uuid.UUID(
                    "550e8400-e29b-41d4-a716-446655440002",
                ),
            ],
        )
        """
        _response = self._raw_client.bulk_retry_notification_logs(request=request, request_options=request_options)
        return _response.data


class AsyncNotificationlogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawNotificationlogsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawNotificationlogsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawNotificationlogsClient
        """
        return self._raw_client

    async def search_notification_logs(
        self,
        *,
        start_date: dt.datetime,
        end_date: dt.datetime,
        page_size: typing.Optional[Pagesize] = None,
        skip: typing.Optional[int] = None,
        notification_event: typing.Optional[str] = OMIT,
        succeeded: typing.Optional[bool] = OMIT,
        org_id: typing.Optional[int] = OMIT,
        paypoint_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[NotificationLog]:
        """
        Search notification logs with filtering and pagination.
          - Start date and end date cannot be more than 30 days apart
          - Either `orgId` or `paypointId` must be provided

        This endpoint requires the `notifications_create` OR `notifications_read` permission.

        Parameters
        ----------
        start_date : dt.datetime
            The start date for the search.

        end_date : dt.datetime
            The end date for the search.

        page_size : typing.Optional[Pagesize]

        skip : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        notification_event : typing.Optional[str]
            The type of notification event to filter by.

        succeeded : typing.Optional[bool]
            Indicates whether the notification was successful.

        org_id : typing.Optional[int]
            The ID of the organization to filter by.

        paypoint_id : typing.Optional[int]
            The ID of the paypoint to filter by.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[NotificationLog]

        Examples
        --------
        import asyncio
        import datetime

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notificationlogs.search_notification_logs(
                page_size=20,
                start_date=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                end_date=datetime.datetime.fromisoformat(
                    "2024-01-31 23:59:59+00:00",
                ),
                org_id=12345,
                notification_event="ActivatedMerchant",
                succeeded=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search_notification_logs(
            start_date=start_date,
            end_date=end_date,
            page_size=page_size,
            skip=skip,
            notification_event=notification_event,
            succeeded=succeeded,
            org_id=org_id,
            paypoint_id=paypoint_id,
            request_options=request_options,
        )
        return _response.data

    async def get_notification_log(
        self, uuid_: uuid.UUID, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NotificationLogDetail:
        """
        Get detailed information for a specific notification log entry.
        This endpoint requires the `notifications_create` OR `notifications_read` permission.

        Parameters
        ----------
        uuid_ : uuid.UUID
            The notification log entry.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotificationLogDetail

        Examples
        --------
        import asyncio
        import uuid

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notificationlogs.get_notification_log(
                uuid_=uuid.UUID(
                    "550e8400-e29b-41d4-a716-446655440000",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_notification_log(uuid_, request_options=request_options)
        return _response.data

    async def retry_notification_log(
        self, uuid_: uuid.UUID, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NotificationLogDetail:
        """
        Retry sending a specific notification.

        **Permissions:** notifications_create

        Parameters
        ----------
        uuid_ : uuid.UUID
            Unique id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotificationLogDetail

        Examples
        --------
        import asyncio
        import uuid

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notificationlogs.retry_notification_log(
                uuid_=uuid.UUID(
                    "550e8400-e29b-41d4-a716-446655440000",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retry_notification_log(uuid_, request_options=request_options)
        return _response.data

    async def bulk_retry_notification_logs(
        self, *, request: BulkRetryRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Retry sending multiple notifications (maximum 50 IDs).
        This is an async process, so use the search endpoint again to check the notification status.

        This endpoint requires the `notifications_create` permission.

        Parameters
        ----------
        request : BulkRetryRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notificationlogs.bulk_retry_notification_logs(
                request=[
                    uuid.UUID(
                        "550e8400-e29b-41d4-a716-446655440000",
                    ),
                    uuid.UUID(
                        "550e8400-e29b-41d4-a716-446655440001",
                    ),
                    uuid.UUID(
                        "550e8400-e29b-41d4-a716-446655440002",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_retry_notification_logs(
            request=request, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

MethodElementSettingsApplePayLanguage = typing.Union[
    typing.Literal[
        "en-US",
        "ar-AB",
        "ca-ES",
        "zh-CN",
        "zh-HK",
        "zh-TW",
        "hr-HR",
        "cs-CZ",
        "da-DK",
        "de-DE",
        "nl-NL",
        "en-AU",
        "en-GB",
        "fi-FI",
        "fr-CA",
        "fr-FR",
        "el-GR",
        "he-IL",
        "hi-IN",
        "hu-HU",
        "id-ID",
        "it-IT",
        "ja-JP",
        "ko-KR",
        "ms-MY",
        "nb-NO",
        "pl-PL",
        "pt-BR",
        "pt-PT",
        "ro-RO",
        "ru-RU",
        "sk-SK",
        "es-MX",
        "es-ES",
        "sv-SE",
        "th-TH",
        "tr-TR",
        "uk-UA",
        "vi-VN",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .billing_data_response import BillingDataResponse
from .contacts import Contacts
from .created_at import CreatedAt
from .dbaname import Dbaname
from .ein import Ein
from .email import Email
from .enrollment_status import EnrollmentStatus
from .entrypointfield import Entrypointfield
from .internal_reference_id import InternalReferenceId
from .last_modified import LastModified
from .legalname import Legalname
from .location_code import LocationCode
from .mcc import Mcc
from .org_parent_name import OrgParentName
from .payee_name import PayeeName
from .remitaddress_1 import Remitaddress1
from .remitaddress_2 import Remitaddress2
from .remitcity import Remitcity
from .remitcountry import Remitcountry
from .remitstate import Remitstate
from .remitzip import Remitzip
from .vendor_number import VendorNumber
from .vendor_summary import VendorSummary
from .vendorstatus import Vendorstatus


class AssociatedVendor(UniversalBaseModel):
    vendor_number: typing_extensions.Annotated[typing.Optional[VendorNumber], FieldMetadata(alias="VendorNumber")] = (
        None
    )
    name_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name1")] = pydantic.Field(
        default=None
    )
    """
    Primary name for vendor.
    """

    name_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name2")] = pydantic.Field(
        default=None
    )
    """
    Secondary name for vendor.
    """

    ein: typing_extensions.Annotated[typing.Optional[Ein], FieldMetadata(alias="EIN")] = None
    phone: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Phone")] = pydantic.Field(
        default=None
    )
    """
    Vendor's phone number.
    """

    email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="Email")] = pydantic.Field(
        default=None
    )
    """
    Vendor's email address.
    """

    address_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Address1")] = pydantic.Field(
        default=None
    )
    """
    Vendor's address.
    """

    address_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Address2")] = pydantic.Field(
        default=None
    )
    """
    Additional line for vendor's address.
    """

    city: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="City")] = pydantic.Field(default=None)
    """
    Vendor's city.
    """

    state: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="State")] = pydantic.Field(
        default=None
    )
    """
    Vendor's state.
    """

    zip: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Zip")] = pydantic.Field(default=None)
    """
    Vendor's postal code.
    """

    country: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Country")] = pydantic.Field(
        default=None
    )
    """
    Vendor's country.
    """

    mcc: typing_extensions.Annotated[typing.Optional[Mcc], FieldMetadata(alias="Mcc")] = None
    location_code: typing_extensions.Annotated[typing.Optional[LocationCode], FieldMetadata(alias="LocationCode")] = (
        None
    )
    contacts: typing_extensions.Annotated[typing.Optional[typing.List[Contacts]], FieldMetadata(alias="Contacts")] = (
        pydantic.Field(default=None)
    )
    """
    Array of objects describing the vendor's contacts.
    """

    billing_data: typing_extensions.Annotated[
        typing.Optional[BillingDataResponse], FieldMetadata(alias="BillingData")
    ] = None
    payment_method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentMethod")] = None
    vendor_status: typing_extensions.Annotated[typing.Optional[Vendorstatus], FieldMetadata(alias="VendorStatus")] = (
        None
    )
    vendor_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="VendorId")] = None
    enrollment_status: typing_extensions.Annotated[
        typing.Optional[EnrollmentStatus], FieldMetadata(alias="EnrollmentStatus")
    ] = None
    summary: typing_extensions.Annotated[typing.Optional[VendorSummary], FieldMetadata(alias="Summary")] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's legal name.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint's DBA name.
    """

    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = pydantic.Field(default=None)
    """
    Paypoint's entryname.
    """

    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    created_date: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedDate")] = None
    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="LastUpdated")] = None
    remit_address_1: typing_extensions.Annotated[
        typing.Optional[Remitaddress1], FieldMetadata(alias="remitAddress1")
    ] = None
    remit_address_2: typing_extensions.Annotated[
        typing.Optional[Remitaddress2], FieldMetadata(alias="remitAddress2")
    ] = None
    remit_city: typing_extensions.Annotated[typing.Optional[Remitcity], FieldMetadata(alias="remitCity")] = None
    remit_state: typing_extensions.Annotated[typing.Optional[Remitstate], FieldMetadata(alias="remitState")] = None
    remit_zip: typing_extensions.Annotated[typing.Optional[Remitzip], FieldMetadata(alias="remitZip")] = None
    remit_country: typing_extensions.Annotated[typing.Optional[Remitcountry], FieldMetadata(alias="remitCountry")] = (
        None
    )
    payee_name_1: typing_extensions.Annotated[typing.Optional[PayeeName], FieldMetadata(alias="payeeName1")] = None
    payee_name_2: typing_extensions.Annotated[typing.Optional[PayeeName], FieldMetadata(alias="payeeName2")] = None
    custom_field_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField1")] = None
    custom_field_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField2")] = None
    customer_vendor_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customerVendorAccount")
    ] = None
    internal_reference_id: typing_extensions.Annotated[
        typing.Optional[InternalReferenceId], FieldMetadata(alias="InternalReferenceId")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="additionalData")
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="externalPaypointID")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .bill_query_record_2 import BillQueryRecord2
from .bill_query_response_summary import BillQueryResponseSummary


class BillQueryResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import (
        BillingData,
        BillItem,
        BillQueryRecord2,
        BillQueryRecord2BillApprovalsItem,
        BillQueryResponse,
        BillQueryResponseSummary,
        GeneralEvents,
        VendorOutData,
    )

    BillQueryResponse(
        summary=BillQueryResponseSummary(
            page_size=20,
            total_2_approval=1,
            totalactive=1,
            total_amount=1.1,
            totalamount_2_approval=1.1,
            totalamountactive=1.1,
            totalamountapproved=1.1,
            totalamountcancel=1.1,
            totalamountdisapproved=1.1,
            totalamountintransit=1.1,
            totalamountoverdue=1.1,
            totalamountpaid=1.1,
            totalamountsent_2_approval=1.1,
            totalapproved=1,
            totalcancel=1,
            totaldisapproved=1,
            totalintransit=1,
            totaloverdue=1,
            total_pages=1,
            totalpaid=1,
            total_records=2,
            totalsent_2_approval=1,
        ),
        records=[
            BillQueryRecord2(
                bill_approvals=[
                    BillQueryRecord2BillApprovalsItem(
                        approved=0,
                        approved_time=datetime.datetime.fromisoformat(
                            "2024-03-13 15:54:27+00:00",
                        ),
                        email="lisandra@example.com",
                        id=34,
                    ),
                    BillQueryRecord2BillApprovalsItem(
                        approved=0,
                        approved_time=datetime.datetime.fromisoformat(
                            "2024-03-13 15:54:27+00:00",
                        ),
                        email="jccastillo@example.com",
                        id=293,
                    ),
                ],
                bill_date=datetime.date.fromisoformat(
                    "2025-03-10",
                ),
                bill_events=[
                    GeneralEvents(
                        description="Created Bill",
                        event_time=datetime.datetime.fromisoformat(
                            "2024-03-13 15:54:26+00:00",
                        ),
                        ref_data="00-45e1c2d8b53b72fafc4082f374e68753-ffea4ba4c2ce63ce-00",
                    ),
                    GeneralEvents(
                        description="Sent to Approval",
                        event_time=datetime.datetime.fromisoformat(
                            "2024-03-13 15:54:28+00:00",
                        ),
                        ref_data="00-086a951822211bc2eb1803ed64db9d4f-0f07e0e8c394e481-00",
                    ),
                ],
                bill_items=[
                    BillItem(
                        item_commodity_code="Commod-MI-2024031926",
                        item_cost=200.0,
                        item_description="Consultation price",
                        item_mode=0,
                        item_product_code="Prod-MI-2024031926",
                        item_product_name="Consultation",
                        item_qty=1,
                        item_tax_amount=0.0,
                        item_tax_rate=0.0,
                        item_total_amount=200.0,
                        item_unit_of_measure="per each",
                    )
                ],
                bill_number="MI-bill-2024031926",
                comments="PAYBILL",
                created_at=datetime.datetime.fromisoformat(
                    "2024-03-13 15:54:26+00:00",
                ),
                discount=0.0,
                due_date=datetime.date.fromisoformat(
                    "2025-03-10",
                ),
                external_paypoint_id="micasa-10",
                frequency="one-time",
                id_bill=6104,
                last_updated=datetime.datetime.fromisoformat(
                    "2024-03-13 10:54:26+00:00",
                ),
                lot_number="LOT123",
                mode=0,
                net_amount=200.0,
                parent_org_id=1001,
                parent_org_name="Fitness Hub",
                paypoint_dbaname="MiCasa Sports",
                paypoint_entryname="micasa",
                paypoint_legalname="MiCasa Sports LLC",
                source="web",
                status=2,
                terms="Net30",
                total_amount=200.0,
                vendor=VendorOutData(
                    address_1="1234 Liberdad St.",
                    address_2="Suite 100",
                    billing_data=BillingData(
                        account_number="12345XXXX",
                        bank_account_function=0,
                        bank_account_holder_name="Elena Gomez",
                        bank_account_holder_type="Business",
                        bank_name="Michigan Savings Bank",
                        id=0,
                        routing_account="072000326",
                        type_account="Checking",
                    ),
                    city="Detroit",
                    country="US",
                    ein="XXXXX6789",
                    email="elenag@industriesexample.com",
                    internal_reference_id=1215,
                    mcc="700",
                    name_1="Gomez-Radulescu Industries",
                    name_2="Elena",
                    phone="517-555-0123",
                    state="MI",
                    vendor_id=8723,
                    vendor_number="MI-vendor-2024031926",
                    vendor_status=1,
                    zip="48201",
                ),
            )
        ],
    )
    """

    summary: typing_extensions.Annotated[typing.Optional[BillQueryResponseSummary], FieldMetadata(alias="Summary")] = (
        pydantic.Field(default=None)
    )
    """
    Summary statistics for the bill query response.
    """

    records: typing_extensions.Annotated[
        typing.Optional[typing.List[BillQueryRecord2]], FieldMetadata(alias="Records")
    ] = pydantic.Field(default=None)
    """
    Array of bill records returned by the query.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .device import Device
from .save_if_success import SaveIfSuccess


class PayMethodCloud(UniversalBaseModel):
    device: typing.Optional[Device] = None
    method: typing.Literal["cloud"] = pydantic.Field(default="cloud")
    """
    Method to use for the transaction. For cloud device transactions, the method is `cloud`.
    """

    save_if_success: typing_extensions.Annotated[
        typing.Optional[SaveIfSuccess], FieldMetadata(alias="saveIfSuccess")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ach_absorb_section import AchAbsorbSection
from .ach_acceptance_element import AchAcceptanceElement
from .ach_fee_section import AchFeeSection
from .ach_pass_through_section import AchPassThroughSection
from .template_element import TemplateElement
from .visible import Visible


class AchService(UniversalBaseModel):
    ach_absorb: typing_extensions.Annotated[typing.Optional[AchAbsorbSection], FieldMetadata(alias="achAbsorb")] = None
    ach_absorb_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="achAbsorb_highPayRange")
    ] = None
    ach_absorb_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="achAbsorb_lowPayRange")
    ] = None
    ach_acceptance: typing_extensions.Annotated[
        typing.Optional[AchAcceptanceElement], FieldMetadata(alias="achAcceptance")
    ] = None
    ach_fees: typing_extensions.Annotated[typing.Optional[AchFeeSection], FieldMetadata(alias="achFees")] = None
    ach_pass_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="achPass_highPayRange")
    ] = None
    ach_pass_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="achPass_lowPayRange")
    ] = None
    ach_pass_through: typing_extensions.Annotated[
        typing.Optional[AchPassThroughSection], FieldMetadata(alias="achPassThrough")
    ] = None
    batch_cutoff_time: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="batchCutoffTime")
    ] = pydantic.Field(default=None)
    """
    Controls how to present the `batchCutoffTime` field on the application. If this field isn't sent, batch cut off time defaults to 5 ET.
    """

    discount_frequency: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="discountFrequency")
    ] = None
    funding_rollup: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="fundingRollup")
    ] = None
    gateway: typing.Optional[TemplateElement] = None
    pdf_template_id: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="pdfTemplateId")
    ] = None
    pricing_plan: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="pricingPlan")] = None
    pricing_type: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="pricingType")] = (
        None
    )
    processor: typing.Optional[TemplateElement] = None
    provider: typing.Optional[TemplateElement] = None
    tier_name: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="tierName")] = None
    visible: typing.Optional[Visible] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.entry import Entry


class CheckCaptureRequest(UniversalBaseModel):
    """
    Request model for check capture processing.
    """

    entry_point: typing_extensions.Annotated[Entry, FieldMetadata(alias="entryPoint")]
    front_image: typing_extensions.Annotated[str, FieldMetadata(alias="frontImage")] = pydantic.Field()
    """
    Base64-encoded image of the front of the check. Must be JPEG or PNG format and less than 1MB. Image must show the entire check clearly with no partial, blurry, or illegible portions.
    """

    rear_image: typing_extensions.Annotated[str, FieldMetadata(alias="rearImage")] = pydantic.Field()
    """
    Base64-encoded image of the back of the check. Must be JPEG or PNG format and less than 1MB. Image must show the entire check clearly with no partial, blurry, or illegible portions.
    """

    check_amount: typing_extensions.Annotated[int, FieldMetadata(alias="checkAmount")] = pydantic.Field()
    """
    Check amount in cents (maximum 32-bit integer value). For example, $125.50 is represented as 12550.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.additional_data import AdditionalData
from ...types.address_addtl_nullable import AddressAddtlNullable
from ...types.address_nullable import AddressNullable
from ...types.contacts_field import ContactsField
from ...types.email import Email
from ...types.location_code import LocationCode
from ...types.mcc import Mcc
from ...types.payee_name import PayeeName
from ...types.remitaddress_1 import Remitaddress1
from ...types.remitaddress_2 import Remitaddress2
from ...types.remitcity import Remitcity
from ...types.remitcountry import Remitcountry
from ...types.remitstate import Remitstate
from ...types.remitzip import Remitzip
from ...types.vendor_ein import VendorEin
from ...types.vendor_name_1 import VendorName1
from ...types.vendor_name_2 import VendorName2
from ...types.vendor_number import VendorNumber
from ...types.vendor_payment_method import VendorPaymentMethod
from ...types.vendor_phone import VendorPhone
from ...types.vendorid import Vendorid
from ...types.vendorstatus import Vendorstatus
from .request_out_authorize_vendor_billing_data import RequestOutAuthorizeVendorBillingData


class RequestOutAuthorizeVendorData(UniversalBaseModel):
    """
    Object containing vendor data.
    """

    vendor_number: typing_extensions.Annotated[typing.Optional[VendorNumber], FieldMetadata(alias="vendorNumber")] = (
        None
    )
    name_1: typing_extensions.Annotated[typing.Optional[VendorName1], FieldMetadata(alias="name1")] = None
    name_2: typing_extensions.Annotated[typing.Optional[VendorName2], FieldMetadata(alias="name2")] = None
    ein: typing.Optional[VendorEin] = None
    phone: typing.Optional[VendorPhone] = None
    email: typing.Optional[Email] = None
    address_1: typing_extensions.Annotated[typing.Optional[AddressNullable], FieldMetadata(alias="address1")] = None
    city: typing.Optional[str] = None
    state: typing.Optional[str] = None
    zip: typing.Optional[str] = None
    country: typing.Optional[str] = None
    mcc: typing.Optional[Mcc] = None
    contacts: typing.Optional[ContactsField] = None
    billing_data: typing_extensions.Annotated[
        typing.Optional[RequestOutAuthorizeVendorBillingData], FieldMetadata(alias="billingData")
    ] = None
    vendor_status: typing_extensions.Annotated[typing.Optional[Vendorstatus], FieldMetadata(alias="vendorStatus")] = (
        None
    )
    remit_address_1: typing_extensions.Annotated[
        typing.Optional[Remitaddress1], FieldMetadata(alias="remitAddress1")
    ] = None
    remit_address_2: typing_extensions.Annotated[
        typing.Optional[Remitaddress2], FieldMetadata(alias="remitAddress2")
    ] = None
    remit_city: typing_extensions.Annotated[typing.Optional[Remitcity], FieldMetadata(alias="remitCity")] = None
    remit_state: typing_extensions.Annotated[typing.Optional[Remitstate], FieldMetadata(alias="remitState")] = None
    remit_zip: typing_extensions.Annotated[typing.Optional[Remitzip], FieldMetadata(alias="remitZip")] = None
    remit_country: typing_extensions.Annotated[typing.Optional[Remitcountry], FieldMetadata(alias="remitCountry")] = (
        None
    )
    customer_vendor_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customerVendorAccount")
    ] = None
    custom_field_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField1")] = None
    custom_field_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField2")] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="additionalData")
    ] = None
    address_2: typing_extensions.Annotated[typing.Optional[AddressAddtlNullable], FieldMetadata(alias="address2")] = (
        None
    )
    internal_reference_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="internalReferenceId")
    ] = None
    location_code: typing_extensions.Annotated[typing.Optional[LocationCode], FieldMetadata(alias="locationCode")] = (
        None
    )
    payee_name_1: typing_extensions.Annotated[typing.Optional[PayeeName], FieldMetadata(alias="payeeName1")] = None
    payee_name_2: typing_extensions.Annotated[typing.Optional[PayeeName], FieldMetadata(alias="payeeName2")] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[VendorPaymentMethod], FieldMetadata(alias="paymentMethod")
    ] = None
    vendor_id: typing_extensions.Annotated[typing.Optional[Vendorid], FieldMetadata(alias="vendorId")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .account_number import AccountNumber
from .bank_account_holder_name import BankAccountHolderName
from .bank_account_holder_type import BankAccountHolderType
from .bank_name import BankName
from .routing_account import RoutingAccount
from .type_account import TypeAccount


class BillingDataResponse(UniversalBaseModel):
    """
    Examples
    --------
    from payabli import BillingDataResponse

    BillingDataResponse(
        id=123456,
        account_id="bank-account-001",
        nickname="Main Checking Account",
        bank_name="Example Bank",
        routing_account="123456789",
        account_number="9876543210",
        type_account="Checking",
        bank_account_holder_name="John Doe",
        bank_account_holder_type="Business",
        bank_account_function=2,
        verified=True,
        status=1,
        services=[],
        default=True,
    )
    """

    id: int = pydantic.Field()
    """
    The bank's ID in Payabli.
    """

    account_id: typing_extensions.Annotated[
        typing.Optional[typing.Optional[typing.Any]], FieldMetadata(alias="accountId")
    ] = None
    nickname: str
    bank_name: typing_extensions.Annotated[BankName, FieldMetadata(alias="bankName")]
    routing_account: typing_extensions.Annotated[RoutingAccount, FieldMetadata(alias="routingAccount")]
    account_number: typing_extensions.Annotated[AccountNumber, FieldMetadata(alias="accountNumber")]
    type_account: typing_extensions.Annotated[TypeAccount, FieldMetadata(alias="typeAccount")]
    bank_account_holder_name: typing_extensions.Annotated[
        BankAccountHolderName, FieldMetadata(alias="bankAccountHolderName")
    ]
    bank_account_holder_type: typing_extensions.Annotated[
        BankAccountHolderType, FieldMetadata(alias="bankAccountHolderType")
    ]
    bank_account_function: typing_extensions.Annotated[int, FieldMetadata(alias="bankAccountFunction")] = (
        pydantic.Field()
    )
    """
    Describes whether the bank account is used for deposits or withdrawals in Payabli:
      - `0`: Deposit
      - `1`: Withdrawal 
      - `2`: Deposit and withdrawal
    """

    verified: bool
    status: int
    services: typing.List[typing.Optional[typing.Any]]
    default: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.cardexp import Cardexp
from ...types.cardholder import Cardholder
from ...types.cardnumber import Cardnumber
from ...types.cardzip import Cardzip
from .request_payment_validate_payment_method_method import RequestPaymentValidatePaymentMethodMethod


class RequestPaymentValidatePaymentMethod(UniversalBaseModel):
    """
    Object describing payment method to use for validation.
    """

    method: RequestPaymentValidatePaymentMethodMethod
    cardnumber: Cardnumber
    cardexp: Cardexp
    cardzip: Cardzip
    card_holder: typing_extensions.Annotated[Cardholder, FieldMetadata(alias="cardHolder")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .is_success import IsSuccess
from .page_identifier import PageIdentifier
from .response_text import ResponseText
from .responsecode import Responsecode
from .responsedata import Responsedata


class PayabliApiResponsePaylinks(UniversalBaseModel):
    is_success: typing_extensions.Annotated[IsSuccess, FieldMetadata(alias="isSuccess")]
    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier")
    ] = None
    response_code: typing_extensions.Annotated[Responsecode, FieldMetadata(alias="responseCode")]
    response_data: typing_extensions.Annotated[Responsedata, FieldMetadata(alias="responseData")] = pydantic.Field()
    """
    The paylink ID or error details.
    """

    response_text: typing_extensions.Annotated[ResponseText, FieldMetadata(alias="responseText")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .apple_pay_id import ApplePayId
from .apple_pay_type import ApplePayType
from .created_at import CreatedAt
from .job_id import JobId
from .job_status import JobStatus
from .last_modified import LastModified
from .organization_id import OrganizationId
from .organization_updates import OrganizationUpdates


class ApplePayOrganizationUpdateData(UniversalBaseModel):
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = None
    id: typing.Optional[ApplePayId] = pydantic.Field(default=None)
    """
    Internal ID for the Apple Pay organization update.
    """

    job_id: typing_extensions.Annotated[typing.Optional[JobId], FieldMetadata(alias="jobId")] = None
    job_status: typing_extensions.Annotated[typing.Optional[JobStatus], FieldMetadata(alias="jobStatus")] = None
    organization_id: typing_extensions.Annotated[
        typing.Optional[OrganizationId], FieldMetadata(alias="organizationId")
    ] = None
    type: typing.Optional[ApplePayType] = pydantic.Field(default=None)
    """
    The record type, in this context it will always be `ApplePayOrganizationUpdate`.
    """

    updated_at: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="updatedAt")] = None
    updates: typing.Optional[OrganizationUpdates] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class CapturePaymentDetails(UniversalBaseModel):
    total_amount: typing_extensions.Annotated[float, FieldMetadata(alias="totalAmount")] = pydantic.Field()
    """
    Total amount to be captured, including the `serviceFee` amount. The amount can't be greater the original
    total amount of the transaction, and can't be more than 15% lower than the original amount.
    """

    service_fee: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="serviceFee")] = (
        pydantic.Field(default=None)
    )
    """
    Service fee to capture for the transaction.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .link_data import LinkData


class BAddress(UniversalBaseModel):
    baddress: typing.Optional[LinkData] = None
    baddress_1: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="baddress1")] = None
    bcity: typing.Optional[LinkData] = None
    bcountry: typing.Optional[LinkData] = None
    bstate: typing.Optional[LinkData] = None
    bzip: typing.Optional[LinkData] = None
    maddress: typing.Optional[LinkData] = None
    maddress_1: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="maddress1")] = None
    mcity: typing.Optional[LinkData] = None
    mcountry: typing.Optional[LinkData] = None
    mstate: typing.Optional[LinkData] = None
    mzip: typing.Optional[LinkData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .key_value_duo import KeyValueDuo
from .notification_standard_request_content_event_type import NotificationStandardRequestContentEventType


class NotificationStandardRequestContent(UniversalBaseModel):
    event_type: typing_extensions.Annotated[
        typing.Optional[NotificationStandardRequestContentEventType], FieldMetadata(alias="eventType")
    ] = pydantic.Field(default=None)
    """
    The notification's event name.
    """

    internal_data: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueDuo]], FieldMetadata(alias="internalData")
    ] = pydantic.Field(default=None)
    """
    Array of pairs key:value to insert in request body to target in **method** = *web*.
    """

    transaction_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="transactionId")] = (
        pydantic.Field(default=None)
    )
    """
    Used internally to reference the entity or object generating the event.
    """

    web_header_parameters: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueDuo]], FieldMetadata(alias="webHeaderParameters")
    ] = pydantic.Field(default=None)
    """
    Array of pairs key:value to insert in header of request to target in **method** = *web*.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .associated_vendor import AssociatedVendor
from .created_at import CreatedAt
from .customer_data import CustomerData
from .dbaname import Dbaname
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .last_modified import LastModified
from .legalname import Legalname
from .org_parent_name import OrgParentName


class VCardRecord(UniversalBaseModel):
    vcard_sent: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="vcardSent")] = pydantic.Field(
        default=None
    )
    """
    When `true`, the vCard has been sent.
    """

    card_token: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="cardToken")] = None
    card_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="cardNumber")] = pydantic.Field(
        default=None
    )
    """
    The vCard number.
    """

    cvc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vCard CVC number.
    """

    expiration_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="expirationDate")] = (
        pydantic.Field(default=None)
    )
    """
    Expiration date in format YYYY-MM-DD. The minimum time to expire is 3 months, maximum is 3 years. If not provided, the default is 6 months.
    """

    status: typing.Optional[str] = None
    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    The vCard amount.
    """

    current_balance: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="currentBalance")] = (
        pydantic.Field(default=None)
    )
    """
    The vCard's current balance.
    """

    expense_limit: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="expenseLimit")] = None
    expense_limit_period: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="expenseLimitPeriod")
    ] = None
    max_number_of_uses: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="maxNumberOfUses")] = None
    current_number_of_uses: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="currentNumberOfUses")
    ] = None
    exact_amount: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="exactAmount")] = None
    mcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    MCC assigned to vCard.
    """

    tcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    TCC assigned to vCard.
    """

    misc_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="misc1")] = pydantic.Field(
        default=None
    )
    """
    Custom field 1.
    """

    misc_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="misc2")] = pydantic.Field(
        default=None
    )
    """
    Custom field 2.
    """

    date_created: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="dateCreated")] = None
    date_modified: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="dateModified")] = (
        None
    )
    associated_vendor: typing_extensions.Annotated[
        typing.Optional[AssociatedVendor], FieldMetadata(alias="associatedVendor")
    ] = None
    associated_customer: typing_extensions.Annotated[
        typing.Optional[CustomerData], FieldMetadata(alias="associatedCustomer")
    ] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint's DBA name.
    """

    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's legal name.
    """

    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's entry name (entrypoint).
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .customer_id import CustomerId
from .customer_number_nullable import CustomerNumberNullable
from .customer_query_records_customer_consent import CustomerQueryRecordsCustomerConsent
from .customer_status import CustomerStatus
from .customer_summary_record import CustomerSummaryRecord
from .dbaname import Dbaname
from .email import Email
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .identifierfields import Identifierfields
from .legalname import Legalname
from .method_query_records import MethodQueryRecords
from .mfa import Mfa
from .mfa_mode import MfaMode
from .org_parent_id import OrgParentId
from .org_parent_name import OrgParentName
from .page_identifier import PageIdentifier
from .shippingaddress import Shippingaddress
from .shippingaddressadditional import Shippingaddressadditional
from .shippingcity import Shippingcity
from .shippingcountry import Shippingcountry
from .shippingstate import Shippingstate
from .shippingzip import Shippingzip
from .subscription_query_records import SubscriptionQueryRecords
from .timezone import Timezone


class CustomerQueryRecords(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import (
        BinData,
        CustomerQueryRecords,
        CustomerQueryRecordsCustomerConsent,
        CustomerQueryRecordsCustomerConsentECommunication,
        CustomerQueryRecordsCustomerConsentSms,
        CustomerSummaryRecord,
        GeneralEvents,
        MethodQueryRecords,
        PaymentDetail,
        QueryPaymentData,
        SubscriptionQueryRecords,
        TransactionQueryRecords,
    )

    CustomerQueryRecords(
        customer_id=4440,
        customer_number="3456-7645A",
        customer_username="myusername",
        customer_status=1,
        company="AA LLC",
        firstname="John",
        lastname="Smith",
        phone="1234567890",
        email="example@email.com",
        address="3245 Main St",
        address_1="STE 900",
        city="Miami",
        state="FL",
        zip="77777",
        country="US",
        shipping_address="123 Walnut St",
        shipping_address_1="STE 900",
        shipping_city="Johnson City",
        shipping_state="TN",
        shipping_zip="37619",
        shipping_country="US",
        balance=1.1,
        time_zone=-5,
        mfa=False,
        mfa_mode=0,
        sn_provider="facebook",
        sn_identifier="6677fgttyudd999",
        sn_data="",
        last_updated=datetime.datetime.fromisoformat(
            "2021-06-16 05:00:00+00:00",
        ),
        created=datetime.datetime.fromisoformat(
            "2021-06-10 05:00:00+00:00",
        ),
        additional_fields={"property1": "string", "property2": "string"},
        identifier_fields=["email"],
        subscriptions=[
            SubscriptionQueryRecords(
                created_at=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
                end_date=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                entrypage_id=0,
                external_paypoint_id="Paypoint-100",
                fee_amount=3.0,
                frequency="monthly",
                id_sub=396,
                last_run=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                last_updated=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
                left_cycles=15,
                method="card",
                net_amount=3762.87,
                next_date=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                parent_org_name="PropertyManager Pro",
                payment_data=QueryPaymentData(
                    payment_details=PaymentDetail(
                        total_amount=100.0,
                    ),
                ),
                paypoint_dbaname="Sunshine Gutters",
                paypoint_entryname="d193cf9a46",
                paypoint_id=255,
                paypoint_legalname="Sunshine Services, LLC",
                plan_id=0,
                source="api",
                start_date=datetime.datetime.fromisoformat(
                    "2025-10-19 00:00:00+00:00",
                ),
                sub_events=[
                    GeneralEvents(
                        description="TransferCreated",
                        event_time=datetime.datetime.fromisoformat(
                            "2023-07-05 22:31:06+00:00",
                        ),
                    )
                ],
                sub_status=1,
                total_amount=103.0,
                total_cycles=24,
                until_cancelled=True,
            )
        ],
        stored_methods=[
            MethodQueryRecords(
                bin="411111",
                bin_data=BinData(
                    bin_matched_length="6",
                    bin_card_brand="Visa",
                    bin_card_type="Credit",
                    bin_card_category="PLATINUM",
                    bin_card_issuer="Bank of Example",
                    bin_card_issuer_country="United States",
                    bin_card_issuer_country_code_a_2="US",
                    bin_card_issuer_country_number="840",
                    bin_card_is_regulated="false",
                    bin_card_use_category="Consumer",
                    bin_card_issuer_country_code_a_3="USA",
                ),
                descriptor="visa",
                exp_date="1227",
                holder_name="Chad Mercia",
                id_pmethod="6edcbb56-9c0e-4003-b3d1-99abf149ba0e",
                last_updated=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
                masked_account="4XXXXXXXX1111",
                method="card",
            )
        ],
        customer_summary=CustomerSummaryRecord(
            numberof_transactions=30,
            recent_transactions=[
                TransactionQueryRecords(
                    entrypage_id=0,
                    fee_amount=1.0,
                    payor_id=1551,
                    paypoint_id=226,
                    settlement_status=2,
                    total_amount=30.22,
                    trans_status=1,
                )
            ],
            total_amount_transactions=1500.0,
            total_net_amount_transactions=1500.0,
        ),
        paypoint_legalname="Sunshine Services, LLC",
        paypoint_dbaname="Sunshine Gutters",
        parent_org_name="PropertyManager Pro",
        parent_org_id=123,
        paypoint_entryname="d193cf9a46",
        pageidentifier="null",
        external_paypoint_id="Paypoint-100",
        customer_consent=CustomerQueryRecordsCustomerConsent(
            e_communication=CustomerQueryRecordsCustomerConsentECommunication(
                status=1,
                updated_at=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
            ),
            sms=CustomerQueryRecordsCustomerConsentSms(
                status=1,
                updated_at=datetime.datetime.fromisoformat(
                    "2022-07-01 15:00:01+00:00",
                ),
            ),
        ),
    )
    """

    customer_id: typing_extensions.Annotated[typing.Optional[CustomerId], FieldMetadata(alias="customerId")] = None
    customer_number: typing_extensions.Annotated[
        typing.Optional[CustomerNumberNullable], FieldMetadata(alias="customerNumber")
    ] = None
    customer_username: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customerUsername")] = (
        pydantic.Field(default=None)
    )
    """
    Username for customer.
    """

    customer_status: typing_extensions.Annotated[
        typing.Optional[CustomerStatus], FieldMetadata(alias="customerStatus")
    ] = None
    company: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Company")] = pydantic.Field(
        default=None
    )
    """
    Company name.
    """

    firstname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Firstname")] = pydantic.Field(
        default=None
    )
    """
    Customer first name.
    """

    lastname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Lastname")] = pydantic.Field(
        default=None
    )
    """
    Customer last name.
    """

    phone: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Phone")] = pydantic.Field(
        default=None
    )
    """
    Customer phone number.
    """

    email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="Email")] = pydantic.Field(
        default=None
    )
    """
    Customer email address.
    """

    address: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Address")] = pydantic.Field(
        default=None
    )
    """
    Customer address.
    """

    address_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Address1")] = pydantic.Field(
        default=None
    )
    """
    Additional line for customer address.
    """

    city: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="City")] = pydantic.Field(default=None)
    """
    Customer city.
    """

    state: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="State")] = pydantic.Field(
        default=None
    )
    """
    Customer state.
    """

    zip: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Zip")] = pydantic.Field(default=None)
    """
    Customer postal code.
    """

    country: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Country")] = pydantic.Field(
        default=None
    )
    """
    Customer country.
    """

    shipping_address: typing_extensions.Annotated[
        typing.Optional[Shippingaddress], FieldMetadata(alias="ShippingAddress")
    ] = None
    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional], FieldMetadata(alias="ShippingAddress1")
    ] = None
    shipping_city: typing_extensions.Annotated[typing.Optional[Shippingcity], FieldMetadata(alias="ShippingCity")] = (
        None
    )
    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="ShippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[typing.Optional[Shippingzip], FieldMetadata(alias="ShippingZip")] = None
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry], FieldMetadata(alias="ShippingCountry")
    ] = None
    balance: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="Balance")] = pydantic.Field(
        default=None
    )
    """
    Customer balance.
    """

    time_zone: typing_extensions.Annotated[typing.Optional[Timezone], FieldMetadata(alias="TimeZone")] = None
    mfa: typing_extensions.Annotated[typing.Optional[Mfa], FieldMetadata(alias="MFA")] = None
    mfa_mode: typing_extensions.Annotated[typing.Optional[MfaMode], FieldMetadata(alias="MFAMode")] = None
    sn_provider: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="snProvider")] = pydantic.Field(
        default=None
    )
    """
    Social network linked to customer. Possible values:
    
    - `facebook`
    
    - `google`
    
    - `twitter`
    
    - `microsoft`
    """

    sn_identifier: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="snIdentifier")] = (
        pydantic.Field(default=None)
    )
    """
    Identifier or token for customer in linked social network.
    """

    sn_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="snData")] = pydantic.Field(
        default=None
    )
    """
    Additional data provided by the social network related to the customer.
    """

    last_updated: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="LastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    Date and time of last update.
    """

    created: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="Created")] = pydantic.Field(
        default=None
    )
    """
    Date and time created.
    """

    additional_fields: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[str]]], FieldMetadata(alias="AdditionalFields")
    ] = pydantic.Field(default=None)
    """
    List of additional custom fields in format key:value.
    """

    identifier_fields: typing_extensions.Annotated[
        typing.Optional[Identifierfields], FieldMetadata(alias="IdentifierFields")
    ] = None
    subscriptions: typing_extensions.Annotated[
        typing.Optional[typing.List[SubscriptionQueryRecords]], FieldMetadata(alias="Subscriptions")
    ] = pydantic.Field(default=None)
    """
    List of subscriptions associated to the customer.
    """

    stored_methods: typing_extensions.Annotated[
        typing.Optional[typing.List[MethodQueryRecords]], FieldMetadata(alias="StoredMethods")
    ] = pydantic.Field(default=None)
    """
    List of payment methods associated to the customer.
    """

    customer_summary: typing_extensions.Annotated[
        typing.Optional[CustomerSummaryRecord], FieldMetadata(alias="customerSummary")
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    Paypoint legal name.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint DBA name.
    """

    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    parent_org_id: typing_extensions.Annotated[typing.Optional[OrgParentId], FieldMetadata(alias="ParentOrgId")] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = None
    pageidentifier: typing.Optional[PageIdentifier] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    customer_consent: typing_extensions.Annotated[
        typing.Optional[CustomerQueryRecordsCustomerConsent], FieldMetadata(alias="customerConsent")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .notification_report_request_content import NotificationReportRequestContent
from .notification_report_request_frequency import NotificationReportRequestFrequency
from .notification_report_request_method import NotificationReportRequestMethod
from .ownerid import Ownerid
from .ownertype import Ownertype
from .statusnotification import Statusnotification


class NotificationReportRequest(UniversalBaseModel):
    """
    Information about the report notification configuration (report-email, report-web).
    """

    content: NotificationReportRequestContent
    frequency: NotificationReportRequestFrequency
    method: NotificationReportRequestMethod = pydantic.Field()
    """
    Automated reporting lets you gather critical reports without manually filtering and exporting the data. Get automated daily, weekly, and monthly report for daily sales, ACH returns, settlements, and more. You can send these reports via email or via webhook. See [Automated Reports](/developers/developer-guides/notifications-and-webhooks-overview#automated-reports) for more.
    """

    owner_id: typing_extensions.Annotated[typing.Optional[Ownerid], FieldMetadata(alias="ownerId")] = None
    owner_type: typing_extensions.Annotated[Ownertype, FieldMetadata(alias="ownerType")]
    status: typing.Optional[Statusnotification] = None
    target: str = pydantic.Field()
    """
    Specify the notification target.
    
    For method=report-email the expected value is a list of email addresses separated by semicolon.
    
    For method=report-web the expected value is a valid and complete URL. Webhooks support only standard HTTP ports: 80, 443, 8080, or 4443.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .query_transfer_summary import QueryTransferSummary
from .transfer_detail_record import TransferDetailRecord


class QueryTransferDetailResponse(UniversalBaseModel):
    records: typing_extensions.Annotated[typing.List[TransferDetailRecord], FieldMetadata(alias="Records")] = (
        pydantic.Field()
    )
    """
    List of transfer detail records
    """

    summary: typing_extensions.Annotated[QueryTransferSummary, FieldMetadata(alias="Summary")] = pydantic.Field()
    """
    Summary of the transfer details query
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .payabli_pages import PayabliPages
from .paypoint_data import PaypointData


class PaypointEntryConfig(UniversalBaseModel):
    entry_comment: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EntryComment")] = None
    entry_logo: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EntryLogo")] = None
    entry_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EntryName")] = None
    entry_pages: typing_extensions.Annotated[
        typing.Optional[typing.List[PayabliPages]], FieldMetadata(alias="EntryPages")
    ] = None
    entry_subtitle: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EntrySubtitle")] = None
    entry_title: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EntryTitle")] = None
    id_entry: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="IdEntry")] = None
    paypoint: typing_extensions.Annotated[typing.Optional[PaypointData], FieldMetadata(alias="Paypoint")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .file_content_ftype import FileContentFtype


class FileContent(UniversalBaseModel):
    """
    Contains details about a file. Max upload size is 30 MB.
    """

    f_content: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="fContent")] = pydantic.Field(
        default=None
    )
    """
    Content of file, Base64-encoded. Ignored if furl is specified. Max upload size is 30 MB.
    """

    filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the attached file.
    """

    ftype: typing.Optional[FileContentFtype] = pydantic.Field(default=None)
    """
    The MIME type of the file (if content is provided)
    """

    furl: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional URL provided to show or download the file remotely
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
